#include <stdlib.h>
#include "blowfish.h"

unsigned char *
Blowfish_cipher_data(bf_ctx_t *ctx,
		     unsigned char *dest, unsigned char *src,
		     size_t len, int mode)
{
	UWORD_32bits left, right;
	unsigned char *p, *s, *d;
	int i;
	size_t buflen;

	buflen = (len / 8 + 1) * 8;
	if ((len % 8) == 0)
		buflen = len;

	s = (unsigned char *)malloc(buflen);
	if (!s)
		return 0;

	/* padding last block */
	d = s + (buflen - 1) / 8;
	for (i = 0; i < 8; i++)
		*d++ = 0;

	p = src;
	d = s;
	for (i = 0; i < buflen; i++)
		*d++ = *p++;

	p = s;
	d = dest;
	for (i = 0; i < (buflen - 1) / 8 + 1; i++) {
		left = ((*p++) << 24);
		left += ((*p++) << 16);
		left += ((*p++) << 8);
		left += (*p++);
		right = ((*p++) << 24);
		right += ((*p++) << 16);
		right += ((*p++) << 8);
		right += (*p++);
		if (mode)
			Blowfish_encipher(ctx, &left, &right);
		else
			Blowfish_decipher(ctx, &left, &right);
		*d++ = left >> 24 & 0xff;
		*d++ = left >> 16 & 0xff;
		*d++ = left >> 8 & 0xff;
		*d++ = left & 0xff;
		*d++ = right >> 24 & 0xff;
		*d++ = right >> 16 & 0xff;
		*d++ = right >> 8 & 0xff;
		*d++ = right & 0xff;
	}

	/* all clear */
	left = right = 0;
	p = s;
	for (i = 0; i < buflen; i++)
		*p++ = 0;

	free(s);
	return dest;
}
