
/*********************blowfish.c*********************/

/* TODO: test with zero length key */
/* TODO: test with a through z as key and plain text */
/* TODO: make this byte order independent */

#include <stdio.h>		/* used for debugging */
#include <stdlib.h>
#ifdef MACINTOSH
   #include <Types.h>		/* FIXME: do we need this? */
#endif

#include "blowfish.h"
#include "bf_tab.h"		/* P-box P-array, S-box  */

#define S(ctx,x,i) (ctx->bf_S[i][x.w.byte##i])
#define bf_F(ctx,x) (((S(ctx,x,0) + S(ctx,x,1)) ^ S(ctx,x,2)) + S(ctx,x,3))
#define ROUND(ctx,a,b,n) (a.word ^= bf_F(ctx,b) ^ ctx->bf_P[n])

inline
void Blowfish_encipher(bf_ctx_t *ctx, UWORD_32bits *xl, UWORD_32bits *xr)
{
  union aword  Xl;
  union aword  Xr;

  Xl.word = *xl;
  Xr.word = *xr;

  Xl.word ^= ctx->bf_P[0];
  ROUND (ctx, Xr, Xl, 1);  ROUND (ctx, Xl, Xr, 2);
  ROUND (ctx, Xr, Xl, 3);  ROUND (ctx, Xl, Xr, 4);
  ROUND (ctx, Xr, Xl, 5);  ROUND (ctx, Xl, Xr, 6);
  ROUND (ctx, Xr, Xl, 7);  ROUND (ctx, Xl, Xr, 8);
  ROUND (ctx, Xr, Xl, 9);  ROUND (ctx, Xl, Xr, 10);
  ROUND (ctx, Xr, Xl, 11); ROUND (ctx, Xl, Xr, 12);
  ROUND (ctx, Xr, Xl, 13); ROUND (ctx, Xl, Xr, 14);
  ROUND (ctx, Xr, Xl, 15); ROUND (ctx, Xl, Xr, 16);
  Xr.word ^= ctx->bf_P[17];

  *xr = Xl.word;
  *xl = Xr.word;
}

void Blowfish_decipher(bf_ctx_t *ctx, UWORD_32bits *xl, UWORD_32bits *xr)
{
   union aword  Xl;
   union aword  Xr;

   Xl.word = *xl;
   Xr.word = *xr;

   Xl.word ^= ctx->bf_P[17];
   ROUND (ctx, Xr, Xl, 16);  ROUND (ctx, Xl, Xr, 15);
   ROUND (ctx, Xr, Xl, 14);  ROUND (ctx, Xl, Xr, 13);
   ROUND (ctx, Xr, Xl, 12);  ROUND (ctx, Xl, Xr, 11);
   ROUND (ctx, Xr, Xl, 10);  ROUND (ctx, Xl, Xr, 9);
   ROUND (ctx, Xr, Xl, 8);   ROUND (ctx, Xl, Xr, 7);
   ROUND (ctx, Xr, Xl, 6);   ROUND (ctx, Xl, Xr, 5);
   ROUND (ctx, Xr, Xl, 4);   ROUND (ctx, Xl, Xr, 3);
   ROUND (ctx, Xr, Xl, 2);   ROUND (ctx, Xl, Xr, 1);
   Xr.word ^= ctx->bf_P[0];

   *xl = Xr.word;
   *xr = Xl.word;
}

/* FIXME: Blowfish_Initialize() ??? */
bf_ctx_t *InitializeBlowfish(UBYTE_08bits key[], short keybytes)
{
  short          i;		/* FIXME: unsigned int, char? */
  short          j;		/* FIXME: unsigned int, char? */
  UWORD_32bits  data;
  UWORD_32bits  datal;
  UWORD_32bits  datar;
  union aword temp;
  bf_ctx_t *ctx;

/*  fprintf (stderr, "0x%x 0x%x ", bf_P[0], bf_P[1]); /* DEBUG */
/*  fprintf (stderr, "%d %d\n", bf_P[0], bf_P[1]); /* DEBUG */


  if ((ctx = (bf_ctx_t *)malloc(sizeof(bf_ctx_t))) == NULL)
    return NULL;

  for (i = 0; i < bf_N + 2; ++i) {
    ctx->bf_P[i] = bf_P[i];
  }
  for (i = 0; i < 4; ++i) {
    for (j = 0; j < 256; ++j) {
      ctx->bf_S[i][j] = bf_S[i][j];
    }
  }

  j = 0;
  for (i = 0; i < bf_N + 2; ++i) {
    temp.word = 0;
    temp.w.byte0 = key[j];
    temp.w.byte1 = key[(j+1)%keybytes];
    temp.w.byte2 = key[(j+2)%keybytes];
    temp.w.byte3 = key[(j+3)%keybytes];
    data = temp.word;
    ctx->bf_P[i] = ctx->bf_P[i] ^ data;
    j = (j + 4) % keybytes;
  }

  datal = 0x00000000;
  datar = 0x00000000;

  for (i = 0; i < bf_N + 2; i += 2) {
    Blowfish_encipher(ctx, &datal, &datar);

    ctx->bf_P[i] = datal;
    ctx->bf_P[i + 1] = datar;
  }

  for (i = 0; i < 4; ++i) {
    for (j = 0; j < 256; j += 2) {

      Blowfish_encipher(ctx, &datal, &datar);
   
      ctx->bf_S[i][j] = datal;
      ctx->bf_S[i][j + 1] = datar;
    }
  }
  return ctx;
}

void FinishBlowfish(bf_ctx_t *ctx)
{
  int i, j;

  for (i = 0; i < bf_N + 2; ++i) {
    ctx->bf_P[i] = 0;
  }
  for (i = 0; i < 4; ++i) {
    for (j = 0; j < 256; ++j) {
      ctx->bf_S[i][j] = 0;
    }
  }
  free(ctx);
}
