#include <stdio.h>
#include <time.h>

#include "blowfish.h"
#include "bf_util.h"

#define BLOCKSIZE 8

int
main(void)
{
	bf_ctx_t *ctx;
	time_t t;
	char *key = "BLOWFISH";
	unsigned char buf1[9], buf2[9];
	unsigned long initcount, enciphercount, deciphercount;

	t = time(NULL);

	/* warm up */
	while (t == time(NULL));
	printf("initialize for 10 seconds\n");
	initcount = 0;
	while (time(NULL) - t < 10) {
		ctx = InitializeBlowfish(key, 2);
		FinishBlowfish(ctx);
		initcount++;
	}
	printf("%lu keys done.\n", initcount);

	t = time(NULL);
	/* warm up */
	while (t == time(NULL));
	printf("encipher in 10 seconds\n");
	enciphercount = 0;
	ctx = InitializeBlowfish(key, 2);
	while (time(NULL) - t < 10) {
		Blowfish_encipher_data(ctx, buf2, buf1, BLOCKSIZE);
		enciphercount++;
	}
	enciphercount *= BLOCKSIZE;
	FinishBlowfish(ctx);
	printf("%lu bytes done.\n", enciphercount);

	t = time(NULL);
	/* warm up */
	while (t == time(NULL));
	printf("decipher in 10 seconds\n");
	deciphercount = 0;
	ctx = InitializeBlowfish(key, 2);
	while (time(NULL) - t < 10) {
		Blowfish_decipher_data(ctx, buf2, buf1, BLOCKSIZE);
		deciphercount++;
	}
	deciphercount *= BLOCKSIZE;
	FinishBlowfish(ctx);
	printf("%lu bytes done.\n", deciphercount);

	printf("init:     %12.1f(keys/sec)\n", (double) initcount / 10.0);
	printf("encipher: %12.1f(bytes/sec)\n", (double) enciphercount / 10.0);
	printf("decipher: %12.1f(bytes/sec)\n", (double) deciphercount / 10.0);
	return 0;
}
