#include <stdio.h>
#include <string.h>
#include <getopt.h>
#ifdef HAVE_PASSED
#include <pwd.h>
#endif
#include <unistd.h>
#include <errno.h>

#include "blowfish.h"
#include "bf_util.h"

extern char *optarg;
extern int optind;

void usage()
{
	printf("bf_crypt: [df] [-p passwd] [inputfile]\n");
	exit(1);
}

int
main(int argc, char *argv[])
{
	bf_ctx_t *ctx;
	char buf1[17], buf2[17];
	char pass[MAXKEYBYTES * 8];
	int mode = 1;
	FILE *ifp = stdin;
	FILE *ofp = stdout;
	int have_pass = 0;
	int ch;

	while ((ch = getopt(argc, argv, "dp:f:o:")) != -1) {
		switch (ch) {
		case 'd':
			mode = 0; /* decrypt */
			break;
		case 'p':
			strncpy(pass, optarg, sizeof(pass));
			pass[MAXKEYBYTES] = 0;
			have_pass = 1;
			break;
		case 'f':
			if ((ifp = fopen(optarg, "r")) == NULL) {
				(void)fprintf(stderr,
					      "bf_crypt: file open error %s\n", optarg, strerror(errno));
				exit(1);
			}
			break;
		case 'o':
			if ((ofp = fopen(optarg, "w")) == NULL) {
				(void)fprintf(stderr,
					      "bf_crypt: file open error %s\n", optarg, strerror(errno));
				exit(1);
			}
			break;
		case '?':
		default:
			usage();
		}
	}

	if (!have_pass) {
#ifdef HAVE_PASSED
		strncpy(pass, getpass("Password: "), sizeof(pass));
		pass[MAXKEYBYTES] = 0;
#else
		usage();
		exit(1);
#endif
	}

	ctx = InitializeBlowfish(pass, strlen(pass));
	/* round blocksize */
	while (fread(buf1, 1, 8, ifp) == 8) {
		if (mode)
			Blowfish_encipher_data(ctx, buf2, buf1, 8);
		else
			Blowfish_decipher_data(ctx, buf2, buf1, 8);
		fwrite(buf2, 1, 8, ofp);
	}
	FinishBlowfish(ctx);
	return 0;
}

